import unittest
from six import iteritems, string_types
from collections import Iterable


class TestBase(unittest.TestCase):
    def setUp(self):
        super(TestBase, self).setUp()
        print('Unnittests: running <%s>.setUp()' % self.__class__.__name__)
        self.fixtures = []

    def tearDown(self):
        super(TestBase, self).tearDown()
        for fixture in self.fixtures:
            del fixture

    @staticmethod
    def checkEqual(list_a, list_b):
        return len(list_a) == len(list_b) and sorted(list_a) == sorted(list_b)

    def assertDictEqual(self, d1, d2, msg=None):  # assertEqual uses for dicts
        for k, v1 in iteritems(d1):
            self.assertIn(k, d2, msg)
            v2 = d2[k]
            if (isinstance(v1, Iterable) and
                    not isinstance(v1, string_types)):
                self.checkEqual(v1, v2)
            else:
                self.assertEqual(v1, v2, msg)
        return True



from six import iteritems
import nomenclate.core.rendering as rendering
import nomenclate.core.processing as processing
import nomenclate.core.errors as exceptions
import nomenclate.core.tokens as tokens
from . import basetest


class TestNomenclativeBase(basetest.TestBase):
    def setUp(self):
        super(TestNomenclativeBase, self).setUp()
        self.nomenclative_valid = processing.Nomenclative('side_location_nameDecoratorVar_childtype_purpose_type')
        self.nomenclative_valid_short = processing.Nomenclative('side_name_type')
        self.nomenclative_invalid = processing.Nomenclative('test_labelside')

        test_values = tokens.TokenAttrList(['side',
                                                   'location',
                                                   'name',
                                                   'decorator',
                                                   'var',
                                                   'childtype',
                                                   'purpose',
                                                   'type'])
        test_values['side'].set('left')
        test_values['location'].set('rear')
        test_values['name'].set('test')
        test_values['decorator'].set('')
        test_values['var'].set('A')
        test_values['childtype'].set('joints')
        test_values['purpose'].set('offset')
        test_values['type'].set('group')
        self.token_test_dict = test_values.to_json()

        self.fixtures.append([self.nomenclative_valid,
                              self.nomenclative_valid_short,
                              self.nomenclative_invalid,
                              self.token_test_dict])


class TestNomenclativeProcessMatches(TestNomenclativeBase):
    def test_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid.raw_formatted_string)
        for token, value in iteritems(test_dict):
            self.nomenclative_valid.add_match(*value)
        self.assertEquals(self.nomenclative_valid.process_matches(),
                          'left_rear_testA_joints_offset_group')

    def test_valid_short(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.raw_formatted_string)
        for token, value in iteritems(test_dict):
            if isinstance(value, str):
                pass
            else:
                self.nomenclative_valid_short.add_match(*value)
        self.assertEquals(self.nomenclative_valid_short.process_matches(),
                          'left_test_group')

    def test_invalid(self):
        self.assertEquals(self.nomenclative_invalid.process_matches(),
                          self.nomenclative_invalid.raw_formatted_string)


class TestNomenclativeAddMatch(TestNomenclativeBase):
    def test_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid.raw_formatted_string)
        for token, value in iteritems(test_dict):
            self.nomenclative_valid.add_match(*value)
        self.assertEquals(self.nomenclative_valid.process_matches(),
                          'left_rear_testA_joints_offset_group')

    def test_short_valid(self):
        test_dict = self.token_test_dict.copy()
        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.raw_formatted_string)
        for token, value in iteritems(test_dict):
            if not isinstance(value, str):
                self.nomenclative_valid_short.add_match(*value)
        self.assertEquals(self.nomenclative_valid_short.process_matches(),
                          'left_test_group')

    def test_overlap(self):
        test_dict = tokens.TokenAttrList(['name', 'side'])
        test_dict['name'].set('left')
        test_dict['side'].set('left')
        test_dict = test_dict.to_json()

        test_overlap = tokens.TokenAttrList(['side_name'])
        test_overlap['side_name'].set('overlapped')
        test_overlap = test_overlap.to_json()

        rendering.InputRenderer._prepend_token_match_objects(test_dict, self.nomenclative_valid_short.raw_formatted_string)
        rendering.InputRenderer._prepend_token_match_objects(test_overlap, self.nomenclative_valid_short.raw_formatted_string)

        for token, value in iteritems(test_dict):
            if not isinstance(value, str):
                self.nomenclative_valid_short.add_match(*value)

        for token, value in iteritems(test_overlap):
            if not isinstance(value, str):
                self.assertRaises(exceptions.OverlapError, self.nomenclative_valid_short.add_match, *value)

    def test_non_regex_match_object(self):
        self.assertEquals(self.nomenclative_invalid.process_matches(),
                          self.nomenclative_invalid.raw_formatted_string)




#!/bin/env python
# OpenSource 2013, Asi Soudai - www.asimation.com and Alex Segal
# All rights reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#

"""
Wrapper above built-in python logging module to help logging to Shell, Maya and Nuke
directly using time-stamp and logging-level to keep the logs clean.

Also, both Maya and Nuke have specific handlers that allow application sepcific logging
using warning and popup messages that are native to those applications.
fatal and critical levels will pop a warning message in Nuke and Maya to make sure
user attention was grabbed when needed.

This code was inspired by Alex Segal. Thank you Alex.

To use:
    log = getLogger( 'mylogger' )
    log.info( 'log something' )

    # Set level to debug
    log = getLogger( 'mylogger', level=DEBUG )
    # OR
    log = getLogger( 'mylogger' )
    log.setLevel( DEBUG )

    # Alart user:
    try:
        ... do something
    except Exception, error :
        log.fatal( "pop up this message if we're in nuke or maya" )
        raise error # re raise the error after we msg the user.
"""

## -----------------------------------------------------------------------------
# Imports
## -----------------------------------------------------------------------------
import sys
import logging

## Inside Maya?
try:
    import maya

    _in_maya = True
except Exception:
    _in_maya = False

## Inside Nuke?
try:
    import nuke

    _in_nuke = True
except Exception:
    _in_nuke = False

## -----------------------------------------------------------------------------
# Globals
## -----------------------------------------------------------------------------
MSG_FORMAT = "%(asctime)s %(name)s %(levelname)s : %(message)s"
DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

# Levels
CRITICAL = 50
FATAL = CRITICAL
ERROR = 40
WARNING = 30
WARN = WARNING
INFO = 20
DEBUG = 10
NOTSET = 0


## -----------------------------------------------------------------------------
# getLogger - Main call to grab a logger instance
## -----------------------------------------------------------------------------
def getLogger(name, shell=True, maya=_in_maya, nuke=_in_nuke, file=None, level=INFO):
    '''
    Get logger - mimicing logging.getLogger() usage.

        name(str)  : logger name
        shell(bol) : output to shell/stdout
        maya(bol)  : output to maya script editor
        nuke(bol)  : output to nuke script editor
        file(str)  : output log into given fullpath-filename
        level(int) : logger level. default to INFO

    Example:
        log = getLogger( 'mylogger' )
        log.info( 'log something' )
    '''
    return Logger(name, shell, maya, nuke, file, level)


## -----------------------------------------------------------------------------
# Logger Class
## -----------------------------------------------------------------------------
class Logger(object):
    def __init__(self, name, shell=True, maya=False, nuke=False, file=None, level=INFO):

        self._name = name
        self._log = logging.getLogger(name)
        self._log.setLevel(level)

        ## SafetyCheck - since logging.getLogger support multi threads, if logger
        ##               was already created, it should also have the handlers too.
        ##               so let's avoid creating handlers again, so we won't get
        ##               double handlers... and make a double eveything.
        if self._log.handlers:
            # Logger already have handlers, no need to add more.
            return

        # Format
        format = logging.Formatter(MSG_FORMAT, DATE_FORMAT)

        # Shell:
        if shell:
            stream_hdlr = ShellHandler()
            stream_hdlr.setFormatter(format)
            self._log.addHandler(stream_hdlr)

        # File:
        if file:
            file_hdlr = logging.FileHandler(file)
            file_hdlr.setFormatter(format)
            self._log.addHandler(file_hdlr)

        # Maya:
        if maya and _in_maya:
            maya_hdlr = MayaHandler()
            maya_hdlr.setFormatter(format)
            self._log.addHandler(maya_hdlr)

        # Nuke:
        if nuke and _in_nuke:
            nuke_hdlr = NukeHandler()
            nuke_hdlr.setFormatter(format)
            self._log.addHandler(nuke_hdlr)

    def __getattr__(self, attr):
        """ Expose builtin logger attributes.
            this won't be needed if I could inherit the logger directly.
            But the logging.getLoggerClass() don't seem to work without jumping
            through to many hoops to make it worth it.

        :param attr: attribute to get
        """
        if hasattr(self._log, attr):
            return getattr(self._log, attr)
        else:
            raise (AttributeError, "No attribute %s" % attr)

    def __repr__(self):
        return "%s(%s Level:%i)" % (self.__class__, self._name, self.level)

    ## LEVELS
    def debug(self, msg):
        self._log.debug(msg)

    def info(self, msg):
        self._log.info(msg)

    def warning(self, msg):
        self._log.warning(msg)

    def error(self, msg):
        self._log.error(msg)

    def fatal(self, msg):
        self._log.fatal(msg)

    def critical(self, msg):
        self._log.critical(msg)


## -----------------------------------------------------------------------------
# ShellHandler
## -----------------------------------------------------------------------------
class ShellHandler(logging.Handler):
    '''
    Shell Handler - emits logs to stdout.
    we'll use sys.__stdout__ to bypass maya and nuke script-editors
    so they won't show double logging.
    '''

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):
        sys.__stdout__.write("%s\n" % self.format(record))


## -----------------------------------------------------------------------------
# MayaHandler
## -----------------------------------------------------------------------------
class MayaHandler(logging.Handler):
    '''
    Maya specific handler - will emit warnings using nuke.warning()
    and critical and fatal levels will popup a warning dialog so artists
    won't miss important errors.
    '''

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):

        # Formated message:
        msg = self.format(record)

        if record.funcName == "warning":
            maya.cmds.warning("\n" + msg)

        elif record.funcName in ["critical", "fatal"]:

            ## Emit stdout print:
            sys.stdout.write("\n" + msg + "\n")

            ## Open dialog if not in batch mode:
            if maya.cmds.about(batch=True) == False:
                maya.cmds.confirmDialog(title="A %s have accure" % record.funcName,
                                        message=record.message,
                                        button=['Dismiss'],
                                        messageAlign="left")

        else:
            sys.stdout.write(msg + "\n")


## -----------------------------------------------------------------------------
# NukeHandler
## -----------------------------------------------------------------------------
class NukeHandler(logging.Handler):
    '''
    Nuke specific handler - will emit warnings using nuke.warning()
    and critical and fatal levels will popup a warning dialog so artists
    won't miss important errors.
    '''

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):

        # Formated message:
        msg = self.format(record)

        if record.funcName == "warning":
            nuke.warning(msg)

        elif record.funcName in ["critical", "fatal"]:
            nuke.error(msg)
            nuke.message(record.message)

        else:
            sys.stdout.write(msg)


## -----------------------------------------------------------------------------
# let's test it... cause mistakes... happens... from time to time.
# __name__
## -----------------------------------------------------------------------------
if __name__ == '__main__':
    log = getLogger("logger_name", shell=True)
    log.setLevel(logging.DEBUG)
    log.debug('debug')
    log.info('info')
    log.warning('warning')
    log.fatal('fatal')



from six import iteritems
from collections import OrderedDict
from six import add_metaclass
import yaml
import os

import nomenclate.settings as settings
from . import errors as errors
from .tools import (
    gen_dict_key_matches
)

MODULE_LOGGER_LEVEL_OVERRIDE = settings.QUIET


class ConfigEntryFormatter(object):
    def format_query_result(self, query_result, query_path, return_type=list, preceding_depth=None):
        """ Formats the query result based on the return type requested.

        :param query_result: (dict or str or list), yaml query result
        :param query_path: (str, list(str)), representing query path
        :param return_type: type, return type of object user desires
        :param preceding_depth: int, the depth to which we want to encapsulate back up config tree
                                    -1 : defaults to entire tree
        :return: (dict, OrderedDict, str, list), specified return type
        """
        if type(query_result) != return_type:
            converted_result = self.format_with_handler(query_result, return_type)
        else:
            converted_result = query_result

        converted_result = self.add_preceding_dict(converted_result, query_path, preceding_depth)
        return converted_result

    def format_with_handler(self, query_result, return_type):
        """ Uses the callable handler to format the query result to the desired return type

        :param query_result: the result value of our query
        :param return_type: desired return type
        :return: type, the query value as the return type requested
        """
        handler = self.get_handler(type(query_result), return_type)
        return handler.format_result(query_result)

    @staticmethod
    def get_handler(query_result_type, return_type):
        """ Find the appropriate return type handler to convert the query result to the desired return type

        :param query_result_type: type, desired return type
        :param return_type: type, actual return type
        :return: callable, function that will handle the conversion
        """
        try:
            return FormatterRegistry.get_by_take_and_return_type(query_result_type, return_type)
        except (IndexError, AttributeError, KeyError):
            msg = 'Could not find function in conversion list for input type %s and return type %s' % \
                  (query_result_type, return_type)
            raise IndexError(msg)

    @staticmethod
    def add_preceding_dict(config_entry, query_path, preceding_depth):
        """ Adds the preceeding config keys to the config_entry to simulate the original full path to the config entry

        :param config_entry: object, the entry that was requested and returned from the config
        :param query_path: (str, list(str)), the original path to the config_entry
        :param preceding_depth: int, the depth to which we are recreating the preceding config keys
        :return: dict, simulated config to n * preceding_depth
        """
        if preceding_depth is None:
            return config_entry

        preceding_dict = {query_path[-1]: config_entry}
        path_length_minus_query_pos = len(query_path) - 1
        preceding_depth = path_length_minus_query_pos - preceding_depth if preceding_depth != -1 else 0

        for index in reversed(range(preceding_depth, path_length_minus_query_pos)):
            preceding_dict = {query_path[index]: preceding_dict}

        return preceding_dict


class ConfigParse(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)
    config_entry_handler = ConfigEntryFormatter()

    def __init__(self, config_filepath='env.yml'):
        """

        :param config_filepath: str, the path to a user specified config, or the nomenclate default
        """
        self.config_filepath = self.resolve_config_file_path(config_filepath)
        self.config_file_contents = None
        self.rebuild_config_cache(self.config_filepath)

    def resolve_config_file_path(self, config_filepath):
        """ Determines whether given path is valid, and if so uses it.
            Otherwise searches both the working directory and nomenclate/core for the specified config file.

        :param config_filepath: str, file path or relative file name within package
        :return: str, resolved full file path to the config file
        """
        search_paths = [config_filepath,
                        os.path.normpath(os.path.join(os.getcwd(), config_filepath)),
                        os.path.normpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), config_filepath))]

        for search_path in search_paths:
            try:
                self.LOG.info('Checking for config file at path %s' % search_path)
                self.validate_config_file(search_path)
                self.LOG.info('File %s is valid.  Returning.' % config_filepath)
                return search_path
            except (IOError, OSError):
                pass

        msg = 'No config file found in current working directory or nomenclate/core and %s is not a valid YAML file'
        self.LOG.warning(msg)
        raise errors.SourceError(msg)

    def rebuild_config_cache(self, config_filepath):
        """ Loads from file and caches all data from the config file in the form of an OrderedDict to self.data

        :param config_filepath: str, the full filepath to the config file
        :return: bool, success status
        """
        self.validate_config_file(config_filepath)
        config_data = None
        try:
            with open(config_filepath, 'r') as f:
                config_data = yaml.load(f)
            items = list(iteritems(config_data))

        except AttributeError:
            items = list(config_data)

        self.config_file_contents = OrderedDict(sorted(items, key=lambda x: x[0], reverse=True))
        self.config_filepath = config_filepath

    def get(self, query_path=None, return_type=list, preceding_depth=None, throw_null_return_error=False):
        """ Traverses the list of query paths to find the data requested

        :param query_path: (list(str), str), list of query path branches or query string
                                             Default behavior: returns list(str) of possible config headers
        :param return_type: (list, str, dict, OrderedDict), desired return type for the data
        :param preceding_depth: int, returns a dictionary containing the data that traces back up the path for x depth
                                     -1: for the full traversal back up the path
                                     None: is default for no traversal
        :param throw_null_return_error: bool, whether or not to throw an error if we get an empty result but no error
        :return: (list, str, dict, OrderedDict), the type specified from return_type
        :raises: exceptions.ResourceNotFoundError: if the query path is invalid
        """
        function_type_lookup = {str: self._get_path_entry_from_string,
                                list: self._get_path_entry_from_list}

        self.LOG.info(
            'config.get() - Trying to find %s in config and return_type %s' % (repr(query_path), return_type))

        if query_path is None:
            return self._default_config(return_type)

        try:
            config_entry = function_type_lookup.get(type(query_path), str)(query_path)
            self.LOG.info('Retrieved config entry:\n%s' % str(config_entry))
            query_result = self.config_entry_handler.format_query_result(config_entry,
                                                                         query_path,
                                                                         return_type=return_type,
                                                                         preceding_depth=preceding_depth)
            self.LOG.info('Converted config entry:\n%s' % str(query_result))

            return query_result
        except IndexError:
            self.LOG.info('IndexError was found...defaulting to return: %s = %r' %
                          (return_type, return_type()))
            return return_type()

    def _get_path_entry_from_string(self, query_string, first_found=True, full_path=False):
        """ Parses a string to form a list of strings that represents a possible config entry header

        :param query_string: str, query string we are looking for
        :param first_found: bool, return first found entry or entire list
        :param full_path: bool, whether to return each entry with their corresponding config entry path
        :return: (Generator((list, str, dict, OrderedDict)), config entries that match the query string
        :raises: exceptions.ResourceNotFoundError
        """
        iter_matches = gen_dict_key_matches(query_string, self.config_file_contents, full_path=full_path)
        try:
            return next(iter_matches) if first_found else iter_matches
        except (StopIteration, TypeError):
            raise errors.ResourceNotFoundError('Could not find search string %s in the config file contents %s' %
                                               (query_string, self.config_file_contents))

    def _get_path_entry_from_list(self, query_path):
        """ Returns the config entry at query path

        :param query_path: list(str), config header path to follow for entry
        :return: (list, str, dict, OrderedDict), config entry requested
        :raises: exceptions.ResourceNotFoundError
        """
        cur_data = self.config_file_contents
        try:
            self.LOG.info('starting path search from list...' % query_path)
            for child in query_path:
                self.LOG.debug(' -> %s' % child)
                cur_data = cur_data[child]
            self.LOG.debug('Found data %s' % cur_data)
            return cur_data
        except (AttributeError, KeyError):
            raise errors.ResourceNotFoundError('Could not find query path %s in the config file contents' %
                                               query_path)

    def _default_config(self, return_type):
        """ Generates a default instance of whatever the type requested was (in case of miss)

        :param return_type: type, type of object requested
        :return: object, instance of return_type
        """
        self.LOG.debug('Returning default for type %s -> %s' % (return_type, repr(return_type())))
        if return_type == list:
            return [k for k in self.config_file_contents]
        return return_type()

    @classmethod
    def validate_config_file(cls, config_filepath):
        """ Validates the filepath to the config.  Detects whether it is a true YAML file + existance

        :param config_filepath: str, file path to the config file to query
        :return: None
        :raises: IOError
        """
        is_file = os.path.isfile(config_filepath)
        if not is_file and os.path.isabs(config_filepath):
            raise IOError('File path %s is not a valid yml, ini or cfg file or does not exist' % config_filepath)

        elif is_file:
            if os.path.getsize(config_filepath) == 0:
                raise IOError('File %s is empty' % config_filepath)

        with open(config_filepath, 'r') as f:
            if yaml.load(f) is None:
                raise IOError('No YAML config was found in file %s' % config_filepath)


class FormatterRegistry(type):
    CONVERSION_TABLE = {}

    def __new__(mcs, name, bases, dct):
        cls = type.__new__(mcs, name, bases, dct)

        extensions = dct.get('converts')
        accepted_input_type = extensions.get('accepted_input_type', None)
        accepted_return_type = extensions.get('accepted_return_type', None)

        if accepted_input_type and accepted_return_type:
            take_exists = mcs.CONVERSION_TABLE.get(accepted_input_type)
            if not take_exists:
                mcs.CONVERSION_TABLE[accepted_input_type] = {}

            mcs.CONVERSION_TABLE[accepted_input_type][accepted_return_type] = cls

        return cls

    @classmethod
    def get_by_take_and_return_type(mcs, input_type, return_type):
        return mcs.CONVERSION_TABLE[input_type][return_type]


@add_metaclass(FormatterRegistry)
class BaseFormatter(object):
    converts = {'accepted_input_type': None,
                'accepted_return_type': None}

    @staticmethod
    def format_result(input):
        raise NotImplementedError


class StringToList(BaseFormatter):
    converts = {'accepted_input_type': str,
                'accepted_return_type': list}

    @staticmethod
    def format_result(input):
        return input.split()


class DictToString(BaseFormatter):
    converts = {'accepted_input_type': dict,
                'accepted_return_type': str}

    @staticmethod
    def format_result(input):
        return ' '.join(list(input))


class DictToList(BaseFormatter):
    converts = {'accepted_input_type': dict,
                'accepted_return_type': list}

    @staticmethod
    def format_result(input):
        """ Always sorted for order
        """
        keys = list(input)
        keys.sort()
        return keys


class DictToOrderedDict(BaseFormatter):
    converts = {'accepted_input_type': dict,
                'accepted_return_type': OrderedDict}

    @staticmethod
    def format_result(input):
        """From: http://stackoverflow.com/questions/13062300/convert-a-dict-to-sorted-dict-in-python
        """
        items = list(iteritems(input))
        return OrderedDict(sorted(items, key=lambda x: x[0]))


class OrderedDictToList(BaseFormatter):
    converts = {'accepted_input_type': OrderedDict,
                'accepted_return_type': list}

    @staticmethod
    def format_result(input):
        """From: http://stackoverflow.com/questions/13062300/convert-a-dict-to-sorted-dict-in-python
        """
        return list(input)


class NoneToDict(BaseFormatter):
    converts = {'accepted_input_type': type(None),
                'accepted_return_type': dict}

    @staticmethod
    def format_result(input):
        """From: http://stackoverflow.com/questions/13062300/convert-a-dict-to-sorted-dict-in-python
        """
        return {}


class ListToString(BaseFormatter):
    converts = {'accepted_input_type': list,
                'accepted_return_type': str}

    @staticmethod
    def format_result(input):
        return ' '.join(input)


class IntToList(BaseFormatter):
    converts = {'accepted_input_type': int,
                'accepted_return_type': list}

    @staticmethod
    def format_result(input):
        return [input]



import nomenclate as nm
import nomenclate.core.tokens as tokens
import nomenclate.core.errors as exceptions
from . import basetest


class TestTokenAttrBase(basetest.TestBase):
    def setUp(self):
        super(TestTokenAttrBase, self).setUp()
        self.nomenclate = nm.Nom()
        self.token_attr_dict_handler = tokens.TokenAttrList(self.nomenclate.token_dict.to_json())
        self.fixtures.append(self.token_attr_dict_handler)


class TestGetTokenAttr(TestTokenAttrBase):
    def test_get_existing(self):
        print(self.token_attr_dict_handler)
        self.assertEquals(self.token_attr_dict_handler.name, tokens.TokenAttr('name', ''))

    def test_get_not_existing(self):
        self.assertRaises(AttributeError, getattr, self.token_attr_dict_handler, 'no')


class TestTokenAttrs(TestTokenAttrBase):
    def test_get(self):
        self.checkEqual(list(self.token_attr_dict_handler.token_attrs),
                        [tokens.TokenAttr('Var', ''),
                         tokens.TokenAttr('purpose', ''),
                         tokens.TokenAttr('type', ''),
                         tokens.TokenAttr('side', ''),
                         tokens.TokenAttr('location', ''),
                         tokens.TokenAttr('childtype', ''),
                         tokens.TokenAttr('name', ''),
                         tokens.TokenAttr('Decorator', '')])


class TestPurge(TestTokenAttrBase):
    def test_default(self):
        handler = tokens.TokenAttrList(self.nomenclate.state)
        handler.purge_tokens()
        self.assertEquals(handler.token_attrs, [])

    def test_input(self):
        handler = tokens.TokenAttrList(self.nomenclate.state)
        handler.purge_tokens([token.token for token in handler.token_attrs])
        self.assertEquals(handler.token_attrs, [])


class TestEq(TestTokenAttrBase):
    def test_not_equal(self):
        other_token_dict_handler = tokens.TokenAttrList(nm.Nom().token_dict.to_json())
        other_token_dict_handler.merge_json({'name': 'bob'})
        self.assertFalse(self.token_attr_dict_handler == other_token_dict_handler)

    def test_equal(self):
        self.assertTrue(self.token_attr_dict_handler == tokens.TokenAttrList(nm.Nom().token_dict.to_json()))

    def test_not_instance(self):
        self.assertFalse(self.token_attr_dict_handler == 5)


class TestStr(TestTokenAttrBase):
    def test_str(self):
        tokens = ['name', 'decorator', 'purpose', 'childtype', 'type', 'location', 'side', 'var']
        for token in tokens:
            self.assertIn(token, str(self.token_attr_dict_handler))



#!/usr/bin/env python
from . import errors as exceptions
import nomenclate.settings as settings


MODULE_LOGGER_LEVEL_OVERRIDE = settings.QUIET


class TokenMatch(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __init__(self, regex_match, substitution, group_name='token'):
        self.match = regex_match.group(group_name)
        self.start = regex_match.start(group_name)
        self.end = regex_match.end(group_name)
        self.sub = str(substitution)

    @property
    def span(self):
        return self.end - self.start

    def adjust_position(self, other, adjust_by_sub_delta=True):
        self._validate_adjuster(other)
        adjustment = other.span - len(other.sub) if adjust_by_sub_delta else other.span
        if adjustment:
            self.LOG.debug('Adjusting %s by %d' % (self, adjustment))
            self._adjust_order(adjustment)

    def _validate_adjuster(self, other):
        if not isinstance(other, self.__class__):
            raise IOError('Only TokenMatch objects are valid inputs to adjust_position')

        other.overlaps(self)

        if self < other:
            raise IndexError('Current TokenMatch is not affected by input TokenMatch\n\t%s\n\t%s' % (repr(self),
                                                                                                     repr(other)))

    def _adjust_order(self, adjust_value):
        self.start -= adjust_value
        self.end -= adjust_value
        self.LOG.debug('New positions are (%d-%d)' % (self.start, self.end))

    def overlaps(self, other):
        if self in other or other in self:
            raise exceptions.OverlapError('Match %s overlaps with an existing match:\n\t%s\n\t%s' % (self.match,
                                                                                                     self, other))
        return True

    def __contains__(self, other):
        try:
            return self.start < other.start < self.end or self.start < other.end < self.end
        except:
            raise NotImplementedError(
                '<%s>.__contains__ does not handle <%s>' % (self.__class__.__name__, type(other)))

    def __eq__(self, other):
        try:
            return (self.start == other.start and self.end == other.end and
                    self.match == other.match and self.sub == other.sub)
        except:
            raise NotImplementedError(
                '<%s>.__eq__ does not handle <%s>' % (self.__class__.__name__, type(other)))

    def __lt__(self, other):
        if isinstance(other, self.__class__):
            return self.end <= other.start
        else:
            raise NotImplementedError

    def __gt__(self, other):
        if isinstance(other, self.__class__):
            return self.start >= other.end
        else:
            raise NotImplementedError

    def __repr__(self):
        return '<%s %s (%d)- [%d:%d] - replacement = %s>' % (self.__class__.__name__,
                                                             self.match, self.span, self.start, self.end, self.sub)

    def __str__(self):
        return '%s:%s' % (self.match, self.sub)


class Nomenclative(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    def __init__(self, input_str):
        self.raw_formatted_string = input_str
        self.token_matches = []

    def process_matches(self):
        build_str = self.raw_formatted_string
        for token_match in self.token_matches:
            # Do not process static token matches
            if token_match.match.startswith('(') or token_match.match.endswith(')'):
                continue

            if token_match.match == build_str[token_match.start:token_match.end]:
                self.LOG.debug('Processing: %s - %s - %s\n\t%s' % (token_match.match,
                                                                   token_match.sub,
                                                                   token_match.sub,
                                                                   build_str))

                build_str = build_str[:token_match.start] + token_match.sub + build_str[token_match.end:]
                self.adjust_other_matches(token_match)
                self.LOG.debug('Processed as:\n\t%s' % build_str)
        return build_str

    def adjust_other_matches(self, adjuster_match):
        for token_match in [token_match for token_match in self.token_matches if token_match != adjuster_match]:
            try:
                token_match.adjust_position(adjuster_match)
            except IndexError:
                pass
        adjuster_match.end = adjuster_match.start + len(adjuster_match.sub)
        adjuster_match.match = adjuster_match.sub

    def add_match(self, regex_match, substitution):
        token_match = TokenMatch(regex_match, substitution)
        try:
            self.validate_match(token_match)
            self.token_matches.append(token_match)
            self.LOG.info('Added match %s' % self.token_matches[-1])
        except (IndexError, exceptions.OverlapError):
            msg = 'Not adding match %s as it conflicts with a preexisting match' % token_match
            self.LOG.warning(msg)
            raise exceptions.OverlapError(msg)

    def validate_match(self, token_match_candidate):
        for token_match in self.token_matches:
            try:
                token_match.overlaps(token_match_candidate)
            except exceptions.OverlapError:
                msg = "Cannot add match %s due to overlap with %s" % (token_match, token_match_candidate)
                self.LOG.error(msg)
                raise exceptions.OverlapError(msg)

    def __str__(self):
        matches = ' ' if not self.token_matches else ' '.join(map(str, self.token_matches))
        return 'format: %s: %s' % (self.raw_formatted_string, matches)




import nomenclate.core.rendering as rendering
import nomenclate.core.renderers as renderers
import nomenclate.core.processing as processing
import nomenclate.core.tokens as tokens
import nomenclate.core.nomenclature as nom
from . import basetest


class TestRenderersBase(basetest.TestBase):
    def setUp(self):
        super(TestRenderersBase, self).setUp()
        self.nomenclative_valid = processing.Nomenclative('side_location_nameDecoratorVar_childtype_purpose_type')
        self.nomenclative_valid_short = processing.Nomenclative('side_name_type')
        self.nomenclative_invalid = processing.Nomenclative('test_labelside')

        self.token_test_dict = {'side': 'left',
                                'location': 'rear',
                                'name': 'test',
                                'decorator': '',
                                'var': 0,
                                'childtype': 'joints',
                                'purpose': 'offset',
                                'type': 'group'}

        self.fixtures.append([self.nomenclative_valid,
                              self.nomenclative_valid_short,
                              self.nomenclative_invalid,
                              self.token_test_dict])


class TestInputRendererBase(TestRenderersBase):
    def setUp(self):
        super(TestInputRendererBase, self).setUp()
        self.ir = rendering.InputRenderer
        self.nom = nom.Nomenclate()
        self.fixtures.append([self.ir, self.nom])

    def set_values(self):
        self.nom.side.set('left')
        self.nom.name.set('testObject')
        self.nom.type.set('locator')
        self.nom.var.set(0)
        self.nom.var.case = 'upper'


class TestInputRendererProcessTokenAugmentations(TestInputRendererBase):
    def test_from_nomenclate_upper(self):
        self.set_values()
        self.nom.side.case = 'upper'
        self.assertEquals(self.nom.get(), 'L_testObjectA_LOC')

    def test_from_nomenclate_lower(self):
        self.set_values()
        print('\n%s\n' % self.nom.state, self.nom.decorator)
        self.nom.side.case = 'lower'
        self.assertEquals(self.nom.get(), 'l_testObjectA_LOC')

    def test_from_upper(self):
        token_attr = tokens.TokenAttr('name', 'test')
        token_attr.case = 'upper'
        self.assertEquals(renderers.RenderBase.process_token_augmentations('test', token_attr),
                          'TEST')

    def test_from_lower(self):
        token_attr = tokens.TokenAttr('name', 'test')
        token_attr.case = 'lower'
        self.assertEquals(renderers.RenderBase.process_token_augmentations('Test', token_attr),
                          'test')

    def test_from_none(self):
        token_attr = tokens.TokenAttr('name', 'test')
        self.assertEquals(renderers.RenderBase.process_token_augmentations('Test', token_attr),
                          'Test')

    def test_from_prefix(self):
        token_attr = tokens.TokenAttr(token='name', label='test')
        token_attr.prefix = 'v'
        self.assertEquals(renderers.RenderBase.process_token_augmentations('Test', token_attr),
                          'vTest')

    def test_from_suffix(self):
        token_attr = tokens.TokenAttr('name', 'test')
        token_attr.suffix = '_r'
        self.assertEquals(renderers.RenderBase.process_token_augmentations('test', token_attr),
                          'test_r')


class TestInputRendererGetAlphanumericIndex(TestInputRendererBase):
    def test_get__get_alphanumeric_index_integer(self):
        self.assertEquals(self.ir._get_alphanumeric_index(0),
                          [0, 'int'])

    def test_get__get_alphanumeric_index_char_start(self):
        self.assertEquals(self.ir._get_alphanumeric_index('a'),
                          [0, 'char_lo'])

    def test_get__get_alphanumeric_index_char_end(self):
        self.assertEquals(self.ir._get_alphanumeric_index('z'),
                          [25, 'char_lo'])

    def test_get__get_alphanumeric_index_char_upper(self):
        self.assertEquals(self.ir._get_alphanumeric_index('B'),
                          [1, 'char_hi'])

    def test_get__get_alphanumeric_index_error(self):
        self.assertRaises(IOError, self.ir._get_alphanumeric_index, 'asdf')


class TestInputRendererCleanupFormattingString(TestInputRendererBase):
    def test_cleanup_format(self):
        self.assertEquals(self.ir.cleanup_formatted_string('test_name _messed __ up LOC'),
                          'test_name_messed_upLOC')


class TestInputRendererGetVariationId(TestInputRendererBase):
    def test_get_variation_id_normal(self):
        self.assertEquals(renderers.RenderVar._get_variation_id(0), 'a')

    def test_get_variation_id_negative(self):
        self.assertEquals(renderers.RenderVar._get_variation_id(-4), '')

    def test_get_variation_id_negative_one(self):
        self.assertEquals(renderers.RenderVar._get_variation_id(-1), '')

    def test_get_variation_id_double_upper(self):
        self.assertEquals(renderers.RenderVar._get_variation_id(1046, capital=True), 'ANG')

    def test_get_variation_id_double_lower(self):
        self.assertEquals(renderers.RenderVar._get_variation_id(1046, capital=False), 'ang')


class TestInputRendererRenderUniqueTokens(TestInputRendererBase):
    def test_all_replaced(self):
        test_values = tokens.TokenAttrList(['var', 'type', 'side', 'version'])
        test_values['var'].set(0)
        test_values['type'].set('locator')
        test_values['side'].set('left')
        test_values['version'].set(5)
        test_values = test_values.to_json()

        self.nom.merge_dict(test_values)
        self.nom.version.padding = 3
        self.nom.version.prefix = 'v'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals({token: test_values[token]['label'] for token in list(test_values)},
                          {'var': 'a', 'type': 'LOC', 'side': 'l', 'version': 'v005'})


    def test_some_replaced(self):
        test_values = tokens.TokenAttrList(['var', 'type', 'side', 'version'])
        test_values['var'].set(0)
        test_values['type'].set('locator')
        test_values['side'].set('left')
        test_values['version'].set(5)
        test_values = test_values.to_json()

        self.nom.merge_dict(test_values)
        self.nom.version.padding = 3
        self.nom.version.prefix = 'v'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {'var': 'A', 'type': 'LOC', 'side': 'l', 'version': 'v005'})


    def test_default_renderer(self):
        test_values = tokens.TokenAttrList(['var',
                                                   'type',
                                                   'side',
                                                   'version',
                                                   'john',
                                                   'purpose'])
        test_values['var'].set(0)
        test_values['type'].set('locator')
        test_values['side'].set('left')
        test_values['version'].set(5)
        test_values['john'].set('six')
        test_values['purpose'].set('hierarchy')

        test_values = test_values.to_json()

        self.nom.merge_dict(test_values)
        self.nom.version.padding = 3
        self.nom.version.prefix = 'v'

        self.nom.format = self.nom.format + '_john'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {'var': 'A',
                           'type': 'LOC',
                           'side': 'l',
                           'version': 'v005',
                           'john': 'six',
                           'purpose': 'hrc'})


    def test_empty(self):
        test_values = {}
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values,
                          {})


    def test_none_replaced(self):
        test_values = tokens.TokenAttrList(['name', 'blah', 'not_me', 'la'])
        test_values['name'].set('test')
        test_values['blah'].set('blah')
        test_values['not_me'].set('not_me')
        test_values['la'].set(5)
        test_values = test_values.to_json()

        self.nom.merge_dict(test_values)
        test_values_unchanged = test_values.copy()
        test_values_unchanged['la'] = '5'
        self.ir.render_unique_tokens(self.nom, test_values)
        self.assertEquals(test_values, test_values_unchanged)



from six import iteritems
import collections
import nomenclate.settings as settings

MODULE_LOGGER_LEVEL_OVERRIDE = settings.QUIET
LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)


class Serializable(object):
    SERIALIZE_ATTRS = []

    def serialize(self):
        return self.to_json()

    def deserialize(self, blob):
        return self.from_json(blob)

    def merge_serialization(self, blob):
        return self.merge_json(blob)

    def to_json(self):
        return {attr: getattr(self, attr) for attr in self.SERIALIZE_ATTRS}

    def merge_json(self, json_blob):
        LOG.info('Merging blob %s to instance %r with serializable attributes %s' % (json_blob,
                                                                                     self,
                                                                                     self.SERIALIZE_ATTRS))
        for search_attr in [attr for attr in self.SERIALIZE_ATTRS if json_blob.get(attr) is not None]:
            json_attr = json_blob[search_attr]
            setattr(self, search_attr, json_attr)
            LOG.info('Successfully merged %s = %s' % (search_attr, json_attr))
        LOG.info('State of instance is now %r' % self)
        return True

    @classmethod
    def from_json(cls, json_blob):
        return cls(**{attr: json_blob[attr] for attr in cls.SERIALIZE_ATTRS if json_blob.get(attr) is not None})


class NomenclateNotifier(object):
    def __init__(self, observer):
        self.observers = []
        self.register_observer(observer)

    def register_observer(self, observer_function):
        LOG.info('Registering observer function %s' % observer_function)
        self.observers.append(observer_function)

    def notify_observer(self, *args, **kwargs):
        for observer_function in self.observers:
            LOG.info('Notifying %s with args %s and kwargs %s' % (observer_function.__name__, args, kwargs))
            observer_function(*args, **kwargs)


def get_string_difference(string1, string2):
    last_found_index = 0
    matches = []
    longer_string = max(string1, string2)
    shorter_string = min(string1, string2)

    for char_index, char in enumerate(shorter_string):
        comparison_index = max(char_index, last_found_index)

        while comparison_index < len(longer_string):
            if longer_string[comparison_index] == char:
                last_found_index = comparison_index
                matches.append([char_index, longer_string[comparison_index], comparison_index])
                break
            comparison_index += 1

    return matches


def combine_dicts(*args, **kwargs):
    """ Combines all arguments (if they are dictionaries) and kwargs to a final dict

    :param args: dict, any dictionaries the user wants combined
    :param kwargs: dict, kwargs.
    :return: dict, compiled dictionary
    """
    dicts = [arg for arg in args if isinstance(arg, dict)]
    LOG.debug('dicts are %s' % str(dicts))
    dicts.append(kwargs)
    super_dict = collections.defaultdict(dict)

    for d in dicts:
        for k, v in iteritems(d):
            if k:
                super_dict[k] = v
    LOG.debug('super dict is %s' % str(dict(super_dict)))
    return dict(super_dict)


def get_keys_containing(search_string, input_dict, default=None, first_found=True):
    """ Searches a dictionary for all keys containing the search string (as long as the keys are not functions) and
        returns a filtered dictionary of only those keys
    
    :param search_string: str, string we are looking for
    :param input_dict: dict, dictionary to search through
    :param default: object, if we find nothing, return this as the default
    :param first_found: bool, return the first found key
    :return: dict, dictionary filled with any matching keys
    """
    output = {}
    for k, v in iteritems(input_dict):
        if search_string in k and not callable(k):
            output[k] = v

    if first_found:
        try:
            output = output[next(iter(output))]
        except StopIteration:
            pass
    return output or default


def gen_dict_key_matches(key, dictionary, _path=None, full_path=False):
    """ Generates a list of sets of (path, value) where path is a list of keys to access the given value in the given
        nested dictionary

    :param key: object, given dictionary key we are looking for
    :param dictionary: dict, query dictionary
    :param _path: list(str), internal use for the current path to output in case of match.
    :param full_path: bool, return value or (path, value))
    :return: Generator[object] or Generator[(list(str), object)], generator of (path, value) or just value
    """
    if _path is None:
        _path = []
    for k, v in iteritems(dictionary):
        _path.append(k)
        if k == key:
            yield (_path, v) if full_path else v
        elif isinstance(v, dict):
            for result in gen_dict_key_matches(key, v, _path):
                yield result


def flatten(it):
    """ Flattens any iterable
        From:
        http://stackoverflow.com/questions/11503065/python-function-to-flatten-generator-containing-another-generator

    :param it: Iterator, iterator to flatten
    :return: Generator, A generator of the flattened values
    """
    for x in it:
        if isinstance(x, collections.Iterable) and not isinstance(x, str):
            for y in flatten(x):
                yield y
        else:
            yield x


def flattenDictToLeaves(d, result=None, index=None):
    if result is None:
        result = []
    if isinstance(d, (list, tuple)):
        for indexB, element in enumerate(d):
            flattenDictToLeaves(element, result, index=indexB)
    elif isinstance(d, dict):
        for key in list(d):
            value = d[key]
            flattenDictToLeaves(value, result, index=None)
    else:
        result.append(d)
    return result



#!/usr/bin/env python
import re
from . import errors as exceptions
import nomenclate.settings as settings

MODULE_LOGGER_LEVEL_OVERRIDE = settings.QUIET


class FormatString(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)

    @property
    def format_order(self):
        return self.processed_format_order

    @format_order.setter
    def format_order(self, format_target):
        if format_target:
            self.processed_format_order = self.get_valid_format_order(format_target)
        else:
            self.processed_format_order = []

    def __init__(self, format_string=""):
        self.LOG.info('Initializing format string with input %r' % format_string)
        self.processed_format_order = []
        self.format_string = format_string
        self.format_order = self.format_string
        self.swap_format(format_string)

    def swap_format(self, format_target):
        try:
            self.format_order = format_target
            self.format_string = format_target
            self.LOG.debug('Successfully set format string: %s and format order: %s' % (self.format_string,
                                                                                        self.format_order))
        except exceptions.FormatError as e:
            msg = "Could not validate input format target %s" % format_target
            self.LOG.error(msg)
            raise e

    @classmethod
    def parse_format_order(cls, format_target):
        """ Dissects the format string and gets the order of the tokens as it finds them l->r
            Splits on camel case or periods/underscores
            Modified version from this:
            http://stackoverflow.com/questions/2277352/python-split-a-string-at-uppercase-letters

        :param format_target: str, format string we want to swap to
        :return: list(str), list of the matching tokens
        """
        cls.LOG.debug('Getting format order from target %s' % repr(format_target))
        try:
            pattern = re.compile(settings.FORMAT_STRING_REGEX)
            return [match.group() for match in pattern.finditer(format_target)]# if None not in match.groups()]
        except TypeError:
            raise exceptions.FormatError('Format string %s is not a valid input type, must be <type str>' %
                                         format_target)

    @classmethod
    def get_valid_format_order(cls, format_target, format_order=None):
        """ Checks to see if the target format string follows the proper style
        """
        cls.LOG.debug('Validating format target %r and parsing for format order %r.' % (format_target, format_order))
        format_order = format_order or cls.parse_format_order(format_target)
        cls.LOG.debug('Resulting format order is %s' % format_order)
        cls.validate_no_token_duplicates(format_order)
        format_target = cls.remove_tokens(format_target, format_order)
        format_target = cls.remove_static_text(format_target)
        cls.validate_separator_characters(format_target)
        cls.validate_matched_parenthesis(format_target)
        cls.LOG.debug('After processing format_target is (should only be valid separator chars): %s' % format_target)
        return format_order

    @staticmethod
    def remove_tokens(format_target, format_order):
        for format_str in format_order:
            format_target = re.sub(format_str, '', format_target, count=1)
        return format_target

    @staticmethod
    def remove_static_text(format_target):
        return re.sub(settings.REGEX_STATIC_TOKEN, '', format_target)

    @classmethod
    def validate_separator_characters(cls, separator_characters):
        cls.LOG.debug('Validating leftover separator characters %s' % separator_characters)
        for char in separator_characters:
            if char not in settings.SEPARATORS:
                msg = "You have specified an invalid format string %s, must be separated by %s." % (
                    separator_characters,
                    settings.SEPARATORS)
                cls.LOG.warning(msg)
                raise exceptions.FormatError(msg)

    @classmethod
    def validate_no_token_duplicates(cls, format_order):
        if len(format_order) != len(list(set([order.lower() for order in format_order]))):
            msg = "Format string has duplicate token names (capitalization insensitive)."
            cls.LOG.warning(msg)
            raise exceptions.FormatError(msg)

    @classmethod
    def validate_matched_parenthesis(cls, format_target):
        """ Adapted from https://stackoverflow.com/questions/6701853/parentheses-pairing-issue

        :param format_order:
        :return:
        """
        iparens = iter('(){}[]<>')
        parens = dict(zip(iparens, iparens))
        closing = parens.values()

        def balanced(astr):
            stack = []
            for c in astr:
                d = parens.get(c, None)
                if d:
                    stack.append(d)
                elif c in closing:
                    if not stack or c != stack.pop():
                        return False
            return not stack

        if format_target:
            if not balanced(format_target):
                msg = "Format string has unmatching parentheses."
                cls.LOG.warning(msg)
                raise exceptions.BalanceError(msg)

    def __str__(self):
        return str(self.format_string)



from .nlog import (
    getLogger,
    DEBUG,
    INFO,
    CRITICAL,
    WARNING,
    ERROR,
    FATAL
)

PACKAGE_LOGGER_LEVEL = CRITICAL

CRITICAL = CRITICAL
INFO = INFO
DEBUG = DEBUG
WARNING = WARNING
ERROR = ERROR
FATAL = FATAL
QUIET = None


def get_module_logger(module_name, module_override_level=None):
    module_logger_level = module_override_level or PACKAGE_LOGGER_LEVEL
    return getLogger(module_name, level=module_logger_level)


# Is not a hard coded (word) and does not end with any non word characters or capitals (assuming camel)

SEPARATORS = r'\(\)\{\}\[\]<>.()_\-'
REGEX_SINGLE_PARENTHESIS = r'(\()|(\))'
REGEX_PARENTHESIS = r'([\(\)]+)'
REGEX_BRACKETS = r'([\{\}]+)'
REGEX_STATIC_TOKEN = '(\(\w+\))'
REGEX_BRACKET_TOKEN = r'(\{\w+\})'
REGEX_TOKEN_SEARCH = r'(?P<token>((?<![a-z\(\)]){TOKEN}(?![0-9]))|((?<=[a-z_\(\)]){TOKEN_CAPITALIZED}(?![0-9])))'
REGEX_ADJACENT_UNDERSCORE = r'(^[\W_]+)|([\W_]+$)'
REGEX_SINGLE_LETTER = r'[a-zA-Z]'
REGEX_TOKEN = r'[A-Za-z0-9][^A-Z\W{SEP}]+'.format(SEP=SEPARATORS)

FORMAT_STRING_REGEX = r'(?:{STATIC})|({TOKEN})|{CHAR}'.format(STATIC=REGEX_STATIC_TOKEN,
                                                              TOKEN=REGEX_TOKEN,
                                                              CHAR=REGEX_SINGLE_LETTER)



from six import iteritems
import unittest
import nomenclate as nm
import nomenclate.core.configurator as config
from . import basetest


class TestNomenclateBase(basetest.TestBase):
    def setUp(self):
        super(TestNomenclateBase, self).setUp()
        self.cfg = config.ConfigParse()
        self.test_format = 'side_location_nameDecoratorVar_childtype_purpose_type'
        self.test_format_b = 'location_nameDecoratorVar_childtype_purpose_type_side'

        self.nom = nm.Nom()

        # Inject our fake config
        self.nom.cfg = self.cfg
        self.nom.side.set('left')
        self.nom.name.set('testObject')
        self.nom.type.set('locator')
        self.nom.var.set(0)
        self.nom.var.case = 'upper'
        self.fixtures = [self.cfg, self.nom, self.test_format_b, self.test_format]

    @property
    def fill_vars(self):
        return {'childtype': 'token',
                'purpose': 'filler',
                'side': 'left',
                'var': 0,
                'location': 'top',
                'type': 'locator',
                'name': 'testObject',
                'decorator': 'joint'}

    @property
    def empty_vars(self):
        return {'var': '',
                'type': '',
                'name': '',
                'location': '',
                'decorator': '',
                'side': '',
                'purpose': '',
                'childtype': ''}

    @property
    def partial_vars(self):
        return {'side': 'left',
                'var': 0,
                'type': 'locator',
                'name': 'testObject'}

    @property
    def missing_partials(self):
        return {'location': '',
                'decorator': '',
                'purpose': '',
                'childtype': ''}


class TestNomenclateState(TestNomenclateBase):
    def test_state_clear(self):
        previous_state = self.nom.state
        self.nom.token_dict.reset()
        self.assertEquals(self.nom.state,
                          {'location': '', 'type': '', 'name': '', 'side': '', 'var': '', 'purpose': '',
                           'decorator': '', 'childtype': ''})
        self.nom.state = previous_state

    @unittest.skip
    def test_state_purge(self):
        previous_state = self.nom.state
        self.nom.token_dict.purge_name_attrs()
        self.assertEquals(self.nom.state, {})
        self.nom.state = previous_state

    def test_state_valid(self):
        self.assertEquals({token: token_attr_dict['label'] for token, token_attr_dict in iteritems(self.nom.state)},
                          {'childtype': '',
                           'decorator': '',
                           'location': '',
                           'name': 'testObject',
                           'purpose': '',
                           'side': 'left',
                           'type': 'locator',
                           'var': 0})


class TestNomenclateResetFromConfig(TestNomenclateBase):
    def test_refresh(self):
        self.assertIsNone(self.nom.reset_from_config())


class TestNomenclateInitializeConfigSettings(TestNomenclateBase):
    pass


class TestNomenclateInitializeFormatOptions(TestNomenclateBase):
    def test_switch_naming_format_from_str(self):
        self.nom.initialize_format_options(self.test_format_b)
        self.assertTrue(self.checkEqual(self.nom.format_order,
                                        ['side', 'location', 'name', 'Decorator', 'Var', 'childtype', 'purpose',
                                         'type']))

        self.nom.initialize_format_options(self.test_format)
        self.assertTrue(self.checkEqual(self.nom.format_order,
                                        ['side', 'location', 'name', 'Decorator', 'Var', 'childtype', 'purpose',
                                         'type']))

    def test_switch_multiple_naming_formats_from_config(self):
        self.nom.initialize_format_options(['naming_formats', 'riggers', 'lee_wolland'])
        self.nom.LOG.info(str(self.nom.format_order))
        self.assertTrue(self.checkEqual(self.nom.format_order,
                                        ['type', 'childtype', 'space', 'purpose', 'name', 'side']))

        self.nom.initialize_format_options(['naming_formats', 'riggers', 'raffaele_fragapane'])
        self.assertTrue(self.checkEqual(self.nom.format_order,
                                        ['name', 'height', 'Side', 'Depth', 'purpose']))

        self.nom.initialize_format_options(self.test_format)


class TestNomenclateSwapFormats(TestNomenclateBase):
    lee_path = ['naming_formats', 'riggers', 'lee_wolland']
    raf_path = ['naming_formats', 'riggers', 'raffaele_fragapane']

    def test_format_switch_same_format(self):
        nom = nm.Nom()
        nom_b = nm.Nom()
        self.assertEquals(nom.tokens, nom_b.tokens)
        nom.format = 'side_location_nameDecoratorVar_childtype_purpose_type'
        self.assertEquals(nom.tokens, nom_b.tokens)

    def test_format_switch_same_tokens(self):
        nom = nm.Nom()
        nom_b = nm.Nom()
        self.assertEquals(nom.tokens, nom_b.tokens)
        nom.format = 'type_purpose_name_childtype_decorator_var_location_side'
        self.assertEquals(nom.tokens, nom_b.tokens)

    def test_switch_multiple_naming_formats_use_initialize_format_options(self):
        self.nom.initialize_format_options(self.lee_path)

        self.nom.LOG.info('(lee) New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))
        self.set_values()
        self.assertEquals(self.nom.get(), 'LOC_hchy_test_l')

        self.nom.initialize_format_options(self.raf_path)

        self.set_raf_values()
        self.assertEquals(self.nom.get(), 'test_TLR_hchy')
        self.nom.LOG.info('%r' % self.nom.get())
        self.nom.LOG.info('(raf) New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.nom.initialize_format_options(self.test_format)

    def test_switch_multiple_naming_formats_set_format(self):
        self.nom.format = self.lee_path
        self.nom.LOG.info('(lee) New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.set_values()
        self.assertEquals(self.nom.get(), 'LOC_hchy_test_l')

        self.nom.format = self.raf_path
        print('\nnom state 3 is %s \n' % self.nom.state)
        self.nom.LOG.info('(raf) New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))
        print('\nnom state 3 is %s \n' % self.nom.state)
        print('\nheight 1 is %r \n' % self.nom.height)
        self.set_raf_values()
        self.assertEquals(self.nom.get(), 'test_TLR_hchy')
        self.nom.LOG.info('%r' % self.nom.get())

        self.nom.initialize_format_options(self.test_format)

    def test_switch_single_format(self):
        self.nom.format = self.lee_path
        self.nom.LOG.info('New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.set_values()
        self.assertEquals(self.nom.get(), 'LOC_hchy_test_l')

        self.nom.format = self.raf_path
        self.nom.LOG.info('(raf) New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.set_raf_values()
        self.assertEquals(self.nom.get(), 'test_TLR_hchy')

    def test_switch_single_char_format(self):
        self.nom.format = 'a'
        self.nom.LOG.info('New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.nom.a = 'john'
        self.assertEquals(self.nom.get(), 'john')

    def test_switch_double_format(self):
        self.nom.format = 'af'
        self.nom.LOG.info('New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.nom.af = 'john'
        self.assertEquals(self.nom.get(), 'john')

    def test_switch_triple_format(self):
        self.nom.format = 'asd'
        self.nom.LOG.info('New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.nom.asd = 'john'
        self.assertEquals(self.nom.get(), 'john')

    def test_switch_single_format_nonsense(self):
        self.nom.format = 'asdf'
        self.nom.LOG.info('New Format order and format %s: %s' % (self.nom.format, self.nom.format_order))

        self.nom.asdf = 'john'
        self.assertEquals(self.nom.get(), 'john')

    def test_switch_single_format_repeat(self):
        try:
            self.nom.format = 'asdf_asdf'
        except nm.core.errors.FormatError:
            pass

    def test_switch_single_format_repeat_different_casing(self):
        try:
            self.nom.format = 'asdf_Asdf'
        except nm.core.errors.FormatError:
            pass

    def set_values(self):
        self.nom.name = 'test'
        self.nom.height = 'top'
        self.nom.side = 'left'
        self.nom.depth = 'rear'
        self.nom.purpose = 'hierarchy'

    def set_raf_values(self):
        print('\n%r\n' % self.nom.height)
        self.nom.height = 'top'
        print('\n%r\n' % self.nom.height)
        self.nom.height.case = 'upper'
        self.nom.depth = 'rear'
        self.nom.depth.case = 'upper'
        self.nom.side.case = 'upper'


class TestNomenclateInitializeOptions(TestNomenclateBase):
    def test_options_stored(self):
        self.nom.CONFIG_OPTIONS = None
        self.nom.initialize_options()
        self.assertIsNotNone(self.nom.CONFIG_OPTIONS)


class TestNomenclateInitializeUiOptions(TestNomenclateBase):
    def test_pass_through(self):
        self.nom.initialize_ui_options()


class TestNomenclateMergeDict(TestNomenclateBase):
    def test_partial_merge(self):
        nom = nm.Nom()
        nom.merge_dict(self.partial_vars)
        _ = {}
        _.update(self.partial_vars)
        _.update(self.missing_partials)
        self.assertDictEqual({token: token_attr_dict['label'] for token, token_attr_dict in iteritems(nom.state)}, _)

    def test_not_Found_in_format_merge(self):
        nom = nm.Nom()
        random = {'weird': 'nope', 'not_here': 'haha', 'foo': 'bar'}
        nom.merge_dict(random)
        random.update(self.empty_vars)
        self.assertDictEqual(nom.state, random)


class TestNomenclateGetFormatOrderFromFormatString(TestNomenclateBase):
    def test_get_format_order(self):
        self.assertEquals(self.nom.format_string_object.parse_format_order(self.test_format),
                          ['side', 'location', 'name', 'Decorator', 'Var', 'childtype', 'purpose', 'type'])


class TestNomenclateGet(TestNomenclateBase):
    def test_get(self):
        self.assertEquals(self.nom.get(), 'l_testObjectA_LOC')

    def test_get_after_change(self):
        previous_state = self.nom.state
        self.nom.location.set('rear')
        self.assertEquals(self.nom.get(), 'l_rr_testObjectA_LOC')
        self.nom.state = previous_state


class TestNomenclateEq(TestNomenclateBase):
    def test_equal(self):
        other = nm.Nom(self.nom)
        self.assertTrue(other == self.nom)

    def test_inequal_one_diff(self):
        other = nm.Nom(self.nom.state)
        other.name = 'ronald'
        self.assertFalse(other == self.nom)

    def test_inequal_multi_diff(self):
        other = nm.Nom(self.nom.state)
        other.name = 'ronald'
        other.var = 'C'
        other.type = 'joint'
        print(other, '\n', self.nom)
        self.assertFalse(other == self.nom)


class TestNomenclateRepr(TestNomenclateBase):
    def test__str__(self):
        self.assertEquals(str(self.nom), 'l_testObjectA_LOC')


class TestNomenclateUnset(TestNomenclateBase):
    def test_non_full(self):
        nom = nm.Nom(self.partial_vars)

        _ = {}
        _.update(self.partial_vars)
        _.update(self.missing_partials)

        self.assertDictEqual({token: token_attr_dict['label'] for token, token_attr_dict in iteritems(nom.state)}, _)
        self.assertDictEqual(nom.empty_tokens, self.missing_partials)

    def test_create_empty(self):
        nom = nm.Nom()
        self.assertDictEqual(nom.state, self.empty_vars)
        self.assertDictEqual(nom.empty_tokens, self.empty_vars)

    def test_full(self):
        nom = nm.Nom(self.fill_vars)
        self.assertDictEqual({token: token_attr_dict['label'] for token, token_attr_dict in iteritems(nom.state)},
                             self.fill_vars)
        self.assertDictEqual(nom.empty_tokens, {})


class TestNomenclateTokens(TestNomenclateBase):
    def test_non_full(self):
        nom = nm.Nom()
        self.checkEqual(nom.tokens, list(self.empty_vars))

    def test_partial(self):
        nom = nm.Nom(self.partial_vars)
        self.checkEqual(nom.tokens, list(self.empty_vars))

    def test_full(self):
        nom = nm.Nom(self.fill_vars)
        self.checkEqual(nom.tokens, list(self.empty_vars))


class TestNomenclateDir(TestNomenclateBase):
    def test_default(self):
        nom = nm.Nom()
        for item in nom.tokens:
            self.assertIn(item.token, dir(nom))

    def test_merge_dict(self):
        nom = nm.Nom()
        random = {'weird': 'nope', 'not_here': 'haha', 'foo': 'bar'}
        nom.merge_dict(random)
        for item in list(random):
            self.assertIn(item, dir(nom))

    def test_swap_format(self):
        nom = nm.Nom()
        nom.format = ['naming_formats', 'riggers', 'raffaele_fragapane']
        for item in nom.tokens:
            self.assertIn(item.token, dir(nom))



import nomenclate
import nomenclate.core.tokens as tokens
import nomenclate.core.errors as exceptions
from . import basetest


class TestTokenAttrBase(basetest.TestBase):
    def setUp(self):
        super(TestTokenAttrBase, self).setUp()
        self.token_attr = tokens.TokenAttr('test_token', 'test_label')
        self.fixtures.append(self.token_attr)


class TestTokenAttrInstantiate(TestTokenAttrBase):
    def test_empty_instantiate(self):
        self.assertEquals(tokens.TokenAttr().label, '')

    def test_valid_instantiate(self):
        self.fixtures.append(tokens.TokenAttr('test', 'test'))

    def test_invalid_instantiate_token_list(self):
        self.assertRaises(exceptions.ValidationError, tokens.TokenAttr, [], 'test')

    def test_invalid_instantiate_token_dict(self):
        self.assertRaises(exceptions.ValidationError, tokens.TokenAttr, {}, 'test')

    def test_valid_instantiate_token(self):
        self.assertEquals(tokens.TokenAttr('test', {}).label, "")

    def test_state(self):
        self.assertEquals(self.token_attr.label, 'test_label')
        self.assertEquals(self.token_attr.token, 'test_token')


class TestTokenAttrSet(TestTokenAttrBase):
    def test_set_invalid(self):
        with self.assertRaises(exceptions.ValidationError):
            tokens.TokenAttr().label = [1]
        with self.assertRaises(exceptions.ValidationError):
            tokens.TokenAttr().label = {1: 1}


class TestTokenAttrGet(TestTokenAttrBase):
    def test_get(self):
        self.assertEquals(self.token_attr.label, 'test_label')

    def test_get_empty(self):
        self.assertEquals(tokens.TokenAttr().label, '')


class TestTokenStr(TestTokenAttrBase):
    def test_label_valid(self):
        token_attr = tokens.TokenAttr('name', 'bob')
        self.assertEquals(str(token_attr), repr(token_attr))

    def test_label_empty(self):
        token_attr = tokens.TokenAttr('name', '')
        self.assertEquals(str(token_attr), repr(token_attr))


class TestTokenRepr(TestTokenAttrBase):
    @staticmethod
    def build_repr(token_attr):
        return '<%s (%s): %r>' % (token_attr.__class__.__name__,
                                 token_attr.token,
                                 token_attr.to_json())

    def test_label_valid(self):
        token_attr = tokens.TokenAttr('name', 'bob')
        self.assertEquals(repr(token_attr), self.build_repr(token_attr))

    def test_label_empty(self):
        token_attr = tokens.TokenAttr('name', '')
        self.assertEquals(repr(token_attr), self.build_repr(token_attr))


class TestTokenToken(TestTokenAttrBase):
    def test_token_valid(self):
        token_attr = tokens.TokenAttr('name', 'bob')
        self.assertEquals(token_attr.token, 'name')

    def test_token_empty(self):
        token_attr = tokens.TokenAttr('', 'bob')
        self.assertEquals(token_attr.token, '')

    def test_token_set(self):
        token_attr = tokens.TokenAttr('', 'bob')
        self.assertEquals(token_attr.token, '')
        token_attr.token = 'name'
        self.assertEquals(token_attr.token, 'name')


class TestTokenLabel(TestTokenAttrBase):
    def test_normal_get(self):
        token_attr = tokens.TokenAttr('name', 'Bob')
        self.assertEquals(token_attr.label, 'Bob')


class TestTokenEq(TestTokenAttrBase):
    def test_equal(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') == tokens.TokenAttr('name', 'Bob'))

    def test_not_equal_label(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') == tokens.TokenAttr('name', 'Fob'))

    def test_not_equal_token(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') == tokens.TokenAttr('fame', 'Bob'))

    def test_non_token(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') == '<TokenAttr name(name):\'Bob\'>')


class TestTokenNe(TestTokenAttrBase):
    def test_equal(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') != tokens.TokenAttr('name', 'Bob'))

    def test_not_equal_label(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') != tokens.TokenAttr('name', 'Fob'))

    def test_not_equal_token(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') != tokens.TokenAttr('fame', 'Bob'))

    def test_non_token(self):
        self.assertRaises(AttributeError, tokens.TokenAttr('name', 'Bob').__ne__, '<TokenAttr name(name):\'Bob\'>')


class TestTokenLe(TestTokenAttrBase):
    def test_equal(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('name', 'Bob'))

    def test_not_equal_label(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('name', 'Fob'))

    def test_not_equal_token(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('fame', 'Bob'))

    def test_non_token(self):
        self.assertRaises(AttributeError, tokens.TokenAttr('name', 'Bob').__le__, '<TokenAttr name(name):\'Bob\'>')


class TestTokenGe(TestTokenAttrBase):
    def test_equal(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('name', 'Bob'))

    def test_not_equal_label(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('name', 'Fob'))

    def test_not_equal_token(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') >= tokens.TokenAttr('fame', 'Bob'))

    def test_non_token(self):
        self.assertRaises(AttributeError, tokens.TokenAttr('name', 'Bob').__ge__, '<TokenAttr name(name):\'Bob\'>')


class TestTokenGt(TestTokenAttrBase):
    def test_equal(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') > tokens.TokenAttr('name', 'Bob'))

    def test_not_equal_label(self):
        self.assertFalse(tokens.TokenAttr('name', 'Bob') > tokens.TokenAttr('name', 'Fob'))

    def test_not_equal_token(self):
        self.assertTrue(tokens.TokenAttr('name', 'Bob') > tokens.TokenAttr('fame', 'Bob'))

    def test_non_token(self):
        self.assertRaises(AttributeError, tokens.TokenAttr('name', 'Bob').__gt__, '<TokenAttr name(name):\'Bob\'>')



import re
import nomenclate.core.processing as processing
import nomenclate.core.errors as exceptions
from . import basetest


class TokenMatchBase(basetest.TestBase):
    def setUp(self):
        super(TokenMatchBase, self).setUp()
        self.regex_custom_group_match = next(re.compile(r'(?P<look>test)').finditer('test'))

        test_re_matches = re.compile(r'(?P<token>te)').finditer('test')
        self.token_match_start = processing.TokenMatch(next(test_re_matches), 'mx', group_name='token')

        test_re_matches = re.compile(r'(?P<token>es)').finditer('test')
        self.token_match_mid = processing.TokenMatch(next(test_re_matches), 'lz', group_name='token')

        test_re_matches = re.compile(r'(?P<token>st)').finditer('test')
        self.token_match_end = processing.TokenMatch(next(test_re_matches), 'fr', group_name='token')

        self.fixtures.extend([self.token_match_start,
                              self.token_match_mid,
                              self.token_match_end])


class TokenMatchInit(TokenMatchBase):
    def test_missing_group(self):
        self.assertRaises(IndexError, processing.TokenMatch, self.regex_custom_group_match, 'marg')

    def test_group_custom(self):
        processing.TokenMatch(self.regex_custom_group_match, 'marg', group_name='look')


class TokenMatchContains(TokenMatchBase):
    def test_does_not_contain(self):
        self.assertFalse(self.token_match_start in self.token_match_end)

    def test_contains(self):
        self.assertTrue(self.token_match_start in self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__contains__, 5)


class TokenMatchEquals(TokenMatchBase):
    def test_equal(self):
        self.assertTrue(self.token_match_start,
                        processing.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertTrue(self.token_match_start,
                        self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__eq__, 5)


class TokenMatchGT(TokenMatchBase):
    def test_equal(self):
        self.assertFalse(self.token_match_start >
                         processing.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertFalse(self.token_match_mid > self.token_match_start)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__gt__, 5)


class TokenMatchLT(TokenMatchBase):
    def test_equal(self):
        self.assertFalse(self.token_match_start <
                         processing.TokenMatch(next(re.compile(r'(?P<token>te)').finditer('test')), 'mx'))

    def test_inequal(self):
        self.assertFalse(self.token_match_start < self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(NotImplementedError, self.token_match_start.__lt__, 5)


class TokenMatchAdjustPosition(TokenMatchBase):
    def test_valid_adjust_alongside(self):
        test_re_matches = re.compile(r'(?P<token>st)').finditer('test')
        token_match_end = processing.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_end(self):
        test_re_matches = re.compile(r'(?P<token>er)').finditer('tester')
        token_match_end = processing.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_double(self):
        test_re_matches = re.compile(r'(?P<token>te)').finditer('lesterte')
        token_match_end = processing.TokenMatch(next(test_re_matches), 'fl', group_name='token')
        orig_start = token_match_end.start
        token_match_end.adjust_position(self.token_match_start, adjust_by_sub_delta=False)
        self.assertEquals(token_match_end.start,
                          orig_start - self.token_match_start.span)

    def test_valid_adjust_no_move(self):
        self.assertRaises(IndexError, self.token_match_start.adjust_position, self.token_match_end)

    def test_invalid_adjust(self):
        self.assertRaises(exceptions.OverlapError, self.token_match_start.adjust_position, self.token_match_mid)

    def test_invalid_type(self):
        self.assertRaises(IOError, self.token_match_start.adjust_position, 5)



import codecs
import os
import sys
from os.path import abspath, dirname, join
from distutils.util import convert_path
from setuptools import setup, find_packages
from setuptools.command.install import install

__author__ = 'Andres Weber'
__author_email__ = 'andresmweber@gmail.com'
__package__ = 'nomenclate'
__url__ = 'https://github.com/andresmweber/%s' % __package__

main_ns = {}
with open(convert_path('%s/version.py' % __package__)) as ver_file:
    exec (ver_file.read(), main_ns)
__version__ = main_ns['__version__']

with codecs.open(join(abspath(dirname(__file__)), 'README.rst'), encoding='utf-8') as readme_file:
    long_description = readme_file.read()

description = 'A tool for generating strings based on a preset naming convention.'

install_requires = [
    'python-dateutil',
    'PyYAML',
    'six'
]

tests_requires = [
    'coverage',
    'coveralls',
    'fixtures',
    'unittest2',
    'mock',
    'nose',
    'pyfakefs',
    'tox'
]

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'
    def run(self):
        tag = os.getenv('GIT_TAG')
        if tag != __version__:
            info = "Git tag: {0} does not match the version of this app: {1}".format(tag, __version__)
            sys.exit(info)

dev_requires = ['twine', 'Sphinx', 'docutils', 'docopt']

setup(
    name=__package__,
    version=main_ns['__version__'],
    packages=find_packages(),
    package_data={'configYML': ['nomenclate/core/*.yml']},
    include_package_data=True,
    url=__url__,
    license='MIT',
    author=__author__,
    author_email=__author_email__,
    description=description,
    long_description="`Online Documentation (ReadTheDocs) <http://nomenclate.readthedocs.io/en/latest/>`_",
    keywords='naming conventions labels config convention name parsing parse',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Documentation :: Sphinx',
        'Topic :: Multimedia :: Graphics :: 3D Modeling',
    ],
    install_requires=install_requires,
    extras_require={
        'tests': tests_requires,
        'dev': dev_requires
    },
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)



#!/usr/bin/env python
from six import add_metaclass, iteritems, moves
import datetime
import dateutil.parser as p
import nomenclate.settings as settings
from . import rendering
from . import errors as exceptions
from .tools import (
    gen_dict_key_matches,
    flatten
)

MODULE_LOGGER_LEVEL_OVERRIDE = settings.QUIET


@add_metaclass(rendering.InputRenderer)
class RenderBase(object):
    LOG = settings.get_module_logger(__name__, module_override_level=MODULE_LOGGER_LEVEL_OVERRIDE)
    token = 'default'

    @classmethod
    def render(cls,
               value,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        """ Default renderer for a token.  It checks the config for a match, if not found it uses the value provided.

        :param value: str, value we are trying to match (or config setting for the token)
        :param token: str, token we are searching for
        :param nomenclate_object: nomenclate.core.nomenclate.Nomenclate, instance of nomenclate object to query
        :param filter_kwargs: any config settings that relate to the token as found from the nomenclate instance
        :return: str, the resulting syntactically rendered string
        """
        if config_query_path == None:
            config_query_path = nomenclate_object.OPTIONS_PATH + [token]
            if use_value_in_query_path:
                config_query_path += [value]

        cls.LOG.info('Attempting to default render %r with value %s and kwargs %s' % (token, value, filter_kwargs))
        config_matches = cls.get_config_match(value, token, config_query_path, return_type, nomenclate_object,
                                              **filter_kwargs)

        options = cls.flatten_input(config_matches, value)
        option = cls.process_criteria(token, options, **filter_kwargs) if options else value
        return cls.process_token_augmentations(option, token_attr=getattr(nomenclate_object, token))

    @classmethod
    def process_token_augmentations(cls, value, token_attr):
        """ Uses any found augmentations from the TokenAttr to augment the final rendered value.  Currently
            this only processes the augmentations:
                TokenAttr().case
                TokenAttr().prefix
                TokenAttr().suffix

        :param value: str, the resulting rendered string from the TokenAttr
        :param token_attr: nomenclate.core.tokens.TokenAttr, the processed TokenAttr to be used to query settings.
        :return: str, final augmented string
        """
        cls.LOG.info('Processing augmentations for token attr %r and applying to value %r' % (token_attr, value))
        value = getattr(str, token_attr.case, str)(value)
        value = '{PREFIX}{VALUE}{SUFFIX}'.format(PREFIX=token_attr.prefix,
                                                 VALUE=value,
                                                 SUFFIX=token_attr.suffix)
        cls.LOG.info('Processed as %r' % value)
        return value

    @classmethod
    def get_config_match(cls,
                         query_string,
                         token,
                         entry_path,
                         return_type,
                         nomenclate_object,
                         **filter_kwargs):
        """ Queries the nomenclate's config data for corresponding entries and filters against the incoming
            filter_kwargs as detailed in cls.process_criteria

        :param query_string: str, string to look for once we find a match in the config
        :param token: str, the given token to search for
        :param entry_path: list(str), the query path to the subsection we are looking for to get a match in the config
        :param return_type: type, the type of return value we want (usually should be <type str>)
        :param nomenclate_object: nomenclate.core.nomenclature.Nomenclate, instance to query against (has config data)
        :return: object, whatever return type was specified
        """
        cls.LOG.info('finding config matches for path %s' % entry_path)
        try:
            return nomenclate_object.cfg.get(entry_path, return_type=return_type)
        except exceptions.ResourceNotFoundError:
            cls.LOG.warning('No entry for token %s - defaulting to current: %s' % (token, query_string))
            return query_string

    @classmethod
    def flatten_input(cls, options, query_string):
        """ Takes a list, dict or str of options and outputs a filtered list
            Behavior list:
                dict: flattens/filters the dict to only key matches based on query_string
                list: just flattens a list just in case it's nested.
                str: returns a list with the string in it

        :param options: list, dict, str, input options to flatten/filter
        :param query_string: str, string we are looking for if the input is a dictionary
        :return: list, flattened list.
        """
        cls.LOG.info('Flattening input %s for matches with %s' % (options, query_string))
        if not isinstance(options, (dict, list)):
            options = [options]
        else:
            if isinstance(options, dict):
                options = list(gen_dict_key_matches(query_string, options))

            options = list(flatten(options))
        cls.LOG.info('Flattened to %s' % options)
        return options

    @classmethod
    def process_criteria(cls, token, options, **filter_kwargs):
        """ Each kwarg passed is considered a filter.  The kwarg is in format <token>_<filter function> and if the
            filter function is found in __builtins__ it uses the filter function and checks the result against
            the kwarg's value.  If it passes the check it is filtered out of the current list of options

        :param token: str, token we are querying
        :param options: list(str), the options to filter with kwargs
        :param filter_kwargs: dict(str: str), dictionary of {<token>_<__builtin__ function>: compare value}
        :return:
        """
        options = list(options)
        criteria_matches = list(options)
        cls.LOG.info('Processing criteria kwargs %s' % filter_kwargs)
        for criteria_function_name, criteria in iteritems(filter_kwargs):
            if not criteria_function_name and not criteria:
                continue
            criteria_function_name = criteria_function_name.replace('%s_' % token, '')

            try:
                builtin_func = getattr(moves.builtins, criteria_function_name)
                cls.LOG.info('Filtering options: %s with criteria: %s' % (options, criteria_function_name))
                criteria_matches = [option for option in options if builtin_func(option) == criteria]
            except AttributeError:
                cls.LOG.warning('Criteria function %r is invalid...skipping' % criteria_function_name)

            if not criteria_matches:
                criteria_matches = [min(options, key=lambda x: abs(builtin_func(x) - criteria))]
        cls.LOG.info('Found criteria matches: %s ...returning first' % criteria_matches)
        return criteria_matches[0] if criteria_matches else options[0]


class RenderDate(RenderBase):
    token = 'date'

    @classmethod
    def render(cls,
               date,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        if date == 'now':
            d = datetime.datetime.now()
        else:
            try:
                d = p.parse(date)
            except ValueError:
                return ''
        date_format = getattr(nomenclate_object, '%s_format' % cls.token, '%Y-%m-%d')
        return d.strftime(date_format)


class RenderVar(RenderBase):
    token = 'var'

    @classmethod
    def render(cls,
               var,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        var_format = filter_kwargs.get('%s_format' % cls.token, 'A')
        if isinstance(var, int):
            var = cls._get_variation_id(var, var_format.isupper())
        return cls.process_token_augmentations(var, token_attr=getattr(nomenclate_object, token))

    @staticmethod
    def _get_variation_id(value, capital=False):
        """ Convert an integer value to a character. a-z then double aa-zz etc
        Args:
            value (int): integer index we're looking up
            capital (bool): whether we convert to capitals or not
        Returns (str): alphanumeric representation of the index
        """
        # Reinforcing type just in case a valid string was entered
        value = int(value)
        base_power = base_start = base_end = 0
        while value >= base_end:
            base_power += 1
            base_start = base_end
            base_end += pow(26, base_power)
        base_index = value - base_start

        # create alpha representation
        alphas = ['a'] * base_power
        for index in range(base_power - 1, -1, -1):
            alphas[index] = chr(int(97 + (base_index % 26)))
            base_index /= 26

        characters = ''.join(alphas)
        return characters.upper() if capital else characters


class RenderLod(RenderVar):
    token = 'lod'


class RenderVersion(RenderBase):
    token = 'version'

    @classmethod
    def render(cls,
               version,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        padding = filter_kwargs.get('%s_padding' % token, 4)
        version_string = '%0{0}d'
        version = version_string.format(padding) % int(version)
        return cls.process_token_augmentations(version, token_attr=getattr(nomenclate_object, token))


class RenderType(RenderBase):
    token = 'type'

    @classmethod
    def render(cls,
               engine_type,
               token,
               nomenclate_object,
               config_query_path=None,
               return_type=list,
               use_value_in_query_path=True,
               **filter_kwargs):
        return super(RenderType, cls).render(engine_type,
                                             cls.token,
                                             nomenclate_object,
                                             config_query_path=nomenclate_object.SUFFIXES_PATH,
                                             return_type=dict,
                                             **filter_kwargs)



import unittest
import itertools
import datetime
import nomenclate.core.nameparser as np
from . import basetest


@unittest.skip
class TestNameparser(basetest.TestBase):
    def setUp(self):
        super(TestNameparser, self).setUp()
        self.fixture = np.NameParser()
        self.test_filenames = ['VGS15_sh004_lgt_v002.ma',
                               'vhcl_chevyTraverse_2018_interior_rig_v06_jf.ma',
                               'nurs_enchant_lilies_hi_cn_flowers_receptacle_a_1002_tg',
                               'HKY_010_lighting_v007.hip',
                               'Will_102_060',
                               'cdf0090_lighting_PROPS_FG_RIM_beauty_v011.1062.exr',
                               'I_HKY_150_L1_v1_graded.1001.jpeg',
                               'us.gcr.io_valued-geode-142207%2Fappengine_default.tar',
                               '12-09-16_theStuff_photo_post1s_b02.psd']

    def tearDown(self):
        super(TestNameparser, self).tearDown()
        del self.fixture

    def test_parse_options(self):
        self.assertEquals(self.fixture.parse_name(self.test_filenames[0])['basename']['match'], 'VGS15_sh_lgt')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[1])['basename']['match'], 'vhcl_chevyTraverse')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[2])['basename']['match'],
                          'nurs_enchant_lilies_hi')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[2])['side']['match'], 'cn')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[3])['basename']['match'], 'HKY')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[4])['basename']['match'], 'Will')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[5])['basename']['match'],
                          'cdf_lighting_PROPS_FG_RIM_beauty')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[6])['basename']['match'], 'I_HKY')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[7])['basename']['match'], 'us')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[8])['basename']['match'],
                          'theStuff_photo_post1s_b')
        self.assertEquals(self.fixture.parse_name(self.test_filenames[8])['version']['version'], 2)
        self.assertEquals(self.fixture.parse_name(self.test_filenames[8])['date']['match'], '12-09-16')

    def test_short_options(self):
        self.assertEqual(self.fixture.get_short('robotsLight8|robotLight2|robotLight2Shape'), 'robotLight2Shape')
        self.assertEqual(self.fixture.get_short('robotLight2Shape'), 'robotLight2Shape')
        self.assertEqual(self.fixture.get_short('robotsLight8|robotLight2|l_arm_up_LOC'), 'l_arm_up_LOC')

    def test_discipline_options_invalid(self):
        tests = ['gus_clothing_v10_aw.ZPR',
                 'Total_Protonic_reversal_v001_(Shane).exr',
                 'jacket_NORM.1004.tif',
                 'jacket_substance_EXPORT.abc',
                 '27_12_2015',
                 'clothing_compiled_maya_v01_aw.mb',
                 'QS_296.ZPR',
                 'r_foreLeg.obj',
                 'samsung_galaxy_s6_rough.stl',
                 'mansur_gavriel_purse_back.stl',
                 'LSC_sh01_v8_Nesquick_SFX_MS_WIP_v3_032415-540p_Quicktime.mov',
                 'IMG_20160509_140103743.jpg',
                 'hippydrome_2014.fbx', 'AM152_FBX.part03.rar',
                 'envelope_RB_v003_weights_groundhog.ma',
                 'envelope_weights_02_unsmoothedJoints.json',
                 'moleV01.001.jpg',
                 'robots8|robot2|robott2Shape']
        for test in tests:
            self.assertIsNone(self.fixture.get_discipline(test))

    def test_discipline_options_valid(self):
        tests = [('char_luchadorA-model1_qt1.mov', 'model'),
                 ('kayJewelersPenguin_5402411_build_penguin_rigPuppet_penguin_v2.ma', 'rig'),
                 ('rig_makeGentooPenguin.mel', 'rig'),
                 ('Nesquik_Light_Sign_Anim_Test-1080p_HD_Quicktime.mov', 'Anim'),
                 ('12302016_004_Nesquik_Light_Sign_Anim_Test.mov', 'Anim'),
                 ('nesquikQuicky_5402876_build_char_quicky_model_quicky_lodA_v10.ma', 'model'),
                 ('icons_MDL_v006_aw.ma', 'MDL')]
        for test, val in tests:
            self.assertEquals(self.fixture.get_discipline(test)['match'], val)

    def test_get_side_options(self):
        # Testing for isolated by underscores
        self._side_runner(['%s_arm_up_LOC',
                           'prefix_%s_part_dir_type',
                           'prefix_part_up_%s'])

    def test_get_side_camel(self):
        self._side_runner(['prefix_part%s_dir_type',
                           'prefix_%sPart_dir_type'])

    def test_get_side_no_side_present(self):
        self.assertEquals(self.fixture.get_side('blah_is_no_type_of_side_LOC'),
                          None)

    def test_side_edge_cases(self):
        tests = [('r_foreLeg.obj', ['right', (0, 2), 'r']),
                 ('jacket_substance_EXPORT.abc', None)]

        for test_string, test_value in tests:
            result = self.fixture.get_side(test_string)
            if result:
                result = [result[val] for val in ['side', 'position_full', 'match']]
            self.assertEquals(result, test_value)

    def _side_runner(self, test_options):
        for side in ['left', 'right']:
            permutations = []
            for abbr in np.NameParser._get_abbreviations(side, 3):
                permutations = itertools.chain(permutations,
                                               np.NameParser._get_casing_permutations(abbr))

            for permutation in permutations:
                for test_option in test_options:
                    # Removing unlikely permutations like LeF: they could be mistaken for camel casing on other words
                    if self.fixture.is_valid_camel(permutation):
                        side_results = self.fixture.get_side(test_option % permutation)
                        if side_results:
                            side_results = [side_results[val] for val in ['side', 'position_full', 'match']]

                        test_index = test_option.find('%s') - 1
                        if permutation[0].islower() and test_option[test_index].isalpha() and test_index != -1:
                            self.assertIsNone(side_results)
                        else:
                            for element in [side, permutation]:
                                print(element, side_results)
                                self.assertIn(element, side_results)

    def test_get_date_specific(self):
        self.assertEquals(self.fixture.get_date('12-09-16_theStuff_photo_post1s_b02.psd')['match'], '12-09-16')

    def test_get_date_options(self):
        test_formats = ['%Y-%m-%d_%H-%M-%S',
                        '%Y-%m-%d-%H-%M-%S',
                        '%Y-%m-%d--%H-%M-%S',
                        '%Y-%m-%d%H-%M-%S',
                        '%Y-%m-%d',
                        '%m_%d_%Y',
                        '%m_%d_%y',
                        '%m%d%y',
                        '%m%d%Y',
                        '%y_%m_%dT%H_%M_%S',
                        '%Y%m%d',
                        '%Y%m%d-%H%M%S',
                        '%Y%m%d-%H%M',
                        '%Y']
        input_time = datetime.datetime(2016, 9, 16, 9, 30, 15)
        orders = ['hotel_minatorOutpost_{DATE}_1.5.mb',
                  'hotel_minatorOutpost_1.5.mb_{DATE}',
                  '{DATE}_hotel_minatorOutpost_1.5.mb',
                  '{DATE}.hotel_minatorOutpost_1.5.mb',
                  'hotel_minator{DATE}_tested_LOC',
                  'hotel_{DATE}minator_tested_LOC',
                  'hotel_blahbergG{DATE}minator_tested_LOC',
                  'hippydrome_{DATE}.fbx',
                  'IMG_{DATE}_140103743.jpg']

        for order in orders:
            for test_format in test_formats:
                result = self.fixture.get_date(order.format(DATE=input_time.strftime(test_format)))
                if result:
                    result = (result['datetime'], result['format'])
                self.assertEqual(result,
                                 (datetime.datetime.strptime(input_time.strftime(test_format), test_format),
                                  test_format))

    def test_valid_camel(self):
        for test in [('aAaa', True), ('Aaaa', False), ('AAAaaaaa', True), ('AAAAaaaAAA', True),
                     ('AaA', True), ('Aa', False), ('AA', False), ('aa', False), ('camel', False),
                     ('camelCase', True), ('CamelCase', True), ('CAMELCASE', False), ('camelcase', False),
                     ('Camelcase', False), ('Case', False), ('camelCamelCase', True)]:
            test, val = test
            print (test, val)
            self.assertEquals(self.fixture.is_valid_camel(test), val)

    def test_get_casing_permutations(self):
        # testing all possible permutations of a simple example
        self.assertEquals([i for i in self.fixture._get_casing_permutations('asdf')],
                          ['asdf', 'Asdf', 'aSdf', 'ASdf',
                           'asDf', 'AsDf', 'aSDf', 'ASDf',
                           'asdF', 'AsdF', 'aSdF', 'ASdF',
                           'asDF', 'AsDF', 'aSDF', 'ASDF'])
        # making sure it gives same results regardless of casing of original word
        self.assertEquals([i for i in self.fixture._get_casing_permutations('AfdS')],
                          [i for i in self.fixture._get_casing_permutations('Afds')])
        # double checking shorter casing permutations...it fucks up on length two currently
        self.assertEquals([i for i in self.fixture._get_casing_permutations('lf')],
                          ['lf', 'Lf', 'lF', 'LF'])

    def test_get_abbrs_options(self):
        self.assertEquals([i for i in self.fixture._get_abbreviations('test', 2)],
                          ['te', 'tes', 'test', 'ts', 'tst', 'tt'])
        self.assertEquals([i for i in self.fixture._get_abbreviations('test')],
                          ['te', 'tes', 'test', 'ts', 'tst', 'tt', 't'])

    def test_get_base_options(self):
        self.assertEquals(self.fixture.get_base('gus_clothing_v10_aw.ZPR')['match'], 'gus_clothing')
        self.assertEquals(self.fixture.get_base('jacket_NORM.1004.tif')['match'], 'jacket_NORM')
        self.assertEquals(self.fixture.get_base('jacket_substance_EXPORT.abc')['match'], 'jacket_substance_EXPORT')
        self.assertEquals(self.fixture.get_base('27_12_2015'), None)
        self.assertEquals(self.fixture.get_base('clothing_compiled_maya_v01_aw.mb')['match'], 'clothing_compiled_maya')
        self.assertEquals(self.fixture.get_base('QS_296.ZPR')['match'], 'QS')
        self.assertEquals(self.fixture.get_base('char_luchadorA-model1_qt1.mov')['match'], 'char_luchadorA-model_qt')
        self.assertEquals(
            self.fixture.get_base('kayJewelersPenguin_5402411_build_penguin_rigPuppet_penguin_v2.ma')['match'],
            'kayJewelersPenguin_5402411_build_penguin_rigPuppet_penguin')
        self.assertEquals(self.fixture.get_base('rig_makeGentooPenguin.mel')['match'], 'rig_makeGentooPenguin')
        self.assertEquals(self.fixture.get_base('r_foreLeg.obj')['match'], 'foreLeg')
        self.assertEquals(self.fixture.get_base('samsung_galaxy_s6_rough.stl')['match'], 'samsung_galaxy_s_rough')
        self.assertEquals(self.fixture.get_base('mansur_gavriel_purse_back.stl')['match'], 'mansur_gavriel_purse_back')
        self.assertEquals(
            self.fixture.get_base('LSC_sh01_v8_Nesquick_SFX_MS_WIP_v3_032415-540p_Quicktime.mov')['match'],
            'LSC_sh')
        self.assertEquals(self.fixture.get_base('Nesquik_Light_Sign_Anim_Test-1080p_HD_Quicktime.mov')['match'],
                          'Nesquik_Light_Sign_Anim_Test-1080p_HD_Quicktime')
        self.assertEquals(self.fixture.get_base('12302016_004_Nesquik_Light_Sign_Anim_Test.mov')['match'],
                          'Nesquik_Light_Sign_Anim_Test')
        self.assertEquals(
            self.fixture.get_base('nesquikQuicky_5402876_build_char_quicky_model_quicky_lodA_v10.ma')['match'],
            'nesquikQuicky_5402876_build_char_quicky_model_quicky_lodA')
        self.assertEquals(self.fixture.get_base('IMG_20160509_140103743.jpg')['match'], 'IMG')
        self.assertEquals(self.fixture.get_base('hippydrome_2014.fbx')['match'], 'hippydrome')
        self.assertEquals(self.fixture.get_base('AM152_FBX.part03.rar')['match'], 'AM152_FBX')
        self.assertEquals(self.fixture.get_base('envelope_RB_v003_weights_groundhog.ma')['match'], 'envelope_RB')
        self.assertEquals(self.fixture.get_base('envelope_weights_02_unsmoothedJoints.json')['match'],
                          'envelope_weights')
        self.assertEquals(self.fixture.get_base('icons_MDL_v006_aw.ma')['match'], 'icons_MDL')
        self.assertEquals(self.fixture.get_base('moleV01.001.jpg')['match'], 'mole')

    def test_get_version_options(self):
        tests = [('gus_clothing_v10_aw.ZPR', (10, 'v10')),
                 ('jacket_NORM.1004.tif', (1004, '1004')),
                 ('jacket_substance_EXPORT.abc', None),
                 ('27_12_2015', None),
                 ('clothing_compiled_maya_v01_aw.mb', (1, 'v01')),
                 ('QS_296.ZPR', (296, '296')),
                 ('char_luchadorA-model1_qt1.mov', (1.1, '1')),
                 ('kayJewelersPenguin_5402411_build_penguin_rigPuppet_penguin_v2.ma', (2, 'v2')),
                 ('rig_makeGentooPenguin.mel', None),
                 ('r_foreLeg.obj', None),
                 ('samsung_galaxy_s6_rough.stl', (6, '6')),
                 ('mansur_gavriel_purse_back.stl', None),
                 ('LSC_sh01_v8_Nesquick_SFX_MS\-_WIP_v3_032415-540p_Quicktime.mov', ('1.8.3', '01')),
                 ('Ne1004squik_Light_Sign_Anim_Test-1080p_HD_Quicktime.mov', None),
                 ('12301121_004_Nesquik_Light_Sign_Anim_Test.mov', (4, '004')),
                 ('nesquikQuicky_5402876_build_char_quicky_model_quicky_lodA_v10.ma', (10, 'v10')),
                 ('IMG_20160509_140103743.jpg', None),
                 ('hippydrome_2014.fbx', None),
                 ('AM152_FBX.part03.rar', (3, '03')),
                 ('envelope_RB_v003_weights_groundhog.ma', (3, 'v003')),
                 ('envelope_weights_02_unsmoothedJoints.json', (2, '02')),
                 ('icons_MDL_v0006_aw.ma', (6, 'v0006')),
                 ('moleV01.001.jpg', (1.1, 'V01')),
                 ('VGS15_sh004_lgt_v002', (4.2, '004'))]

        for test, value in tests:
            test_result = self.fixture.get_version(test)
            if test_result is not None:
                version = test_result.get('version', None) or test_result.get('compound_version')
                match = test_result.get('match', None)
                try:
                    match = test_result.get('compound_matches')[0].get('match')
                except (IndexError, TypeError):
                    pass
                print (test_result)
                self.assertEquals((version, match), value)
            else:
                self.assertEquals(test_result, value)

    def test_get_udim_options(self):
        tests = [('gus_clothing_v10_aw.ZPR', None),
                 ('jacket_NORM.1004.tif', 1004),
                 ('jacket_NORM1004.tif', 1004),
                 ('jacket_NORM1004poop.tif', None),  # Deemed a shitty case, not handling it.
                 ('jacket_substance_EXPORT.abc', None),
                 ('27_12_2015', None),
                 ('QS_296.ZPR', None),
                 ('Nesquik_Light_Sign_Anim_Test-1080p_HD_Quicktime.mov', None),
                 ('IMG_20160509_140103743.jpg', None)]
        for test_string, test_value in tests:
            result = self.fixture.get_udim(test_string)
            if result:
                result = result['match_int']
            self.assertEquals(result, test_value)
